#include "PackageTableSortFilterProxyModel.h"
#include "PackageTable.h"
#include "Package.h"

PackageTable::SortFilterProxyModel::SortFilterProxyModel
   (StateFilter  stateFilter,
    bool         groupsFilter,
    YumIO*       yumIO,
    QObject*     parent)
   : HSortFilterProxyModel(NAME, parent),
     mode(FILTER),
     filter(true),
     stateFilter(stateFilter),
     groupsFilter(groupsFilter),
     yumIO(yumIO)
{
}

void PackageTable::SortFilterProxyModel::setMode(Mode  mode)
{
   if (this->mode != mode) {
      this->mode = mode;
      in.clear();
      clear();
   }
}

void PackageTable::SortFilterProxyModel::setFilter(bool  filter)
{
   if (this->filter != filter) {
      this->filter = filter;
      clear();
   }
}

bool PackageTable::SortFilterProxyModel::getFilter ()
{
   return filter;
}

void PackageTable::SortFilterProxyModel::setFilterRegExp
   (const QRegExp&  regExp)
{
   if (regExp != filterRegExp()) {
      in.clear();
   }

   HSortFilterProxyModel::setFilterRegExp(regExp);

   int  row = 0;
   for (; row < sourceModel()->rowCount(); row++) {
      filterAcceptsRow(row, QModelIndex());
   }
}

void PackageTable::SortFilterProxyModel::setStateFilter
   (StateFilter  stateFilter)
{
   if (this->stateFilter != stateFilter) {
      this->stateFilter = stateFilter;
      in.clear();
      clear();
   }
}

void PackageTable::SortFilterProxyModel::setGroupsFilter(bool  groupsFilter)
{
   if (this->groupsFilter != groupsFilter) {
      this->groupsFilter = groupsFilter;
      in.clear();
      clear();
   }
}

bool PackageTable::SortFilterProxyModel::filterAcceptsRow
   (int                 source_row, 
    const QModelIndex&  source_parent) const
{
   if (!filter) return true;

   QModelIndex      ix  = sourceModel()->index(source_row, NAME, source_parent);
   Package::Entry*  pkg = sourceModel()->data(ix, RawValue)
                             .value<Package::Entry*>();

   if (in.contains(pkg)) {
      return in[pkg];
   }

   bool  result = acceptRow(pkg, source_row, source_parent);
   in[pkg] = result;
   return result;
}

bool PackageTable::SortFilterProxyModel::acceptRow
   (Package::Entry*     pkg,
    int                 source_row,
    const QModelIndex&  source_parent) const
{
   // state filter

   {
      using namespace Package;
      switch (stateFilter) {
      case UPDATABLE:   if (pkg->state != OUT_OF_DATE)   return false; break;
      case INSTALLABLE: if (pkg->state != NOT_INSTALLED) return false; break;
      case REMOVABLE:   if (pkg->state == NOT_INSTALLED) return false; break;
      default:                                                         break;
      }
   }

   // groups filter

   if (!groupsFilter) {
      if (pkg->kind == Package::GROUP) {
         return false;
      }
   }

   // string filter or search

   switch (mode) {
   case FILTER:
      if (!filterRegExp().isEmpty()) {
         QString  key;

         int  column;
         for (column = NAME; column < NUM_COLUMNS; column++) {
            QModelIndex  ix = sourceModel()->index(source_row, column,
                                                   source_parent);
            if (!key.isEmpty()) key.append(" ");
            key.append(sourceModel()->data(ix, Qt::DisplayRole).toString());
         }

         if (!key.contains(filterRegExp())) return false;
      }
      break;
   case SEARCH:
      if (!filterRegExp().isEmpty()) {
         switch (pkg->kind) {
         case Package::SUITE:
         case Package::PACKAGE:
            if (!pkg->name.contains(filterRegExp())
                && !pkg->verrel.contains(filterRegExp())
                && !pkg->repo.contains(filterRegExp())
                && !pkg->arch.contains(filterRegExp())
                && !pkg->sizeText.contains(filterRegExp())
                && !pkg->summary.contains(filterRegExp())
                && !yumIO->getDescription(*pkg).contains(filterRegExp())
                && !yumIO->getGroup(*pkg).contains(filterRegExp())
                && !yumIO->getVendor(*pkg).contains(filterRegExp())
                && !yumIO->getBuildTime(*pkg).contains(filterRegExp())
                && !yumIO->getInstallTime(*pkg).contains(filterRegExp())
                && !yumIO->getBinaryRpm(*pkg).contains(filterRegExp())
                && !yumIO->getSourceRpm(*pkg).contains(filterRegExp())
                && !yumIO->getLicense(*pkg).contains(filterRegExp())
                && !yumIO->getPackager(*pkg).contains(filterRegExp())
                && !yumIO->getUrl(*pkg).contains(filterRegExp())
                && !yumIO->getRelocations(*pkg).contains(filterRegExp())
                && !yumIO->getProvides(*pkg).contains(filterRegExp())
                && !yumIO->getRequires(*pkg).contains(filterRegExp())
                && !yumIO->getConflicts(*pkg).contains(filterRegExp())
                && !yumIO->getObsoletes(*pkg).contains(filterRegExp())
                && !yumIO->getFiles(*pkg).contains(filterRegExp())
                && !yumIO->getChangeLog(*pkg).contains(filterRegExp())) {
               return false;
            }
            break;
         case Package::GROUP:
            if (!pkg->name.contains(filterRegExp())
                && !yumIO->getRequiredGroups(*pkg).contains(filterRegExp())
                && !yumIO->getDefaultMetaPkgs(*pkg).contains(filterRegExp())
                && !yumIO->getOptionalMetaPkgs(*pkg).contains(filterRegExp())
                && !yumIO->getMandatoryPkgs(*pkg).contains(filterRegExp())
                && !yumIO->getDefaultPkgs(*pkg).contains(filterRegExp())
                && !yumIO->getOptionalPkgs(*pkg).contains(filterRegExp())
                && !yumIO->getConditionalPkgs(*pkg).contains(filterRegExp())) {
               return false;
            }
            break;
         default:
            return false;
         }
      }
      break;
   }

   return true;
}

Package::Size  PackageTable::SortFilterProxyModel::sizeValue (QString  str)
{
   long long int  unitsMultiplier;

   if (str.endsWith("Gb")) {
      unitsMultiplier = 1024*1024*1024;
      str.chop(2);
   } else if (str.endsWith("Mb")) {
      unitsMultiplier = 1024*1024;
      str.chop(2);
   } else if (str.endsWith("Kb")) {
      unitsMultiplier = 1024;
      str.chop(2);
   } else if (str.endsWith("b")) {
      unitsMultiplier = 1;
      str.chop(1);
   } else {
      return 0;
   }

   return (Package::Size)(str.toDouble() * unitsMultiplier);
}

bool PackageTable::SortFilterProxyModel::lessThan
   (const QModelIndex&  left,
    const QModelIndex&  right) const
{
   switch (left.column()) {
   case ACTION:
   case STATE:
   case KIND:
      if (left.model() == NULL)  return false;
      if (right.model() == NULL) return false;
      {
         QVariant l = left.model()->data(left, RawValue);;
         QVariant r = right.model()->data(right, RawValue);
         if (l.toInt() < r.toInt()) {
            return true;
         } else if (l.toInt() == r.toInt()) {
            return lessThan(createSecondarySortIndex(left),
                            createSecondarySortIndex(right));
         } else {
            return false;
         }
      }
   case SIZE:
      if (left.model() == NULL)  return false;
      if (right.model() == NULL) return false;
      {
         QVariant       l    = left.model()->data(left, Qt::DisplayRole);
         QVariant       r    = right.model()->data(right, Qt::DisplayRole);
         Package::Size  lval = sizeValue(l.toString());
         Package::Size  rval = sizeValue(r.toString());
         if (lval < rval) {
            return true;
         } else if (lval == rval) {
            return lessThan(createSecondarySortIndex(left),
                            createSecondarySortIndex(right));
         } else {
            return false;
         }
      }
   default:
      return HSortFilterProxyModel::lessThan(left, right);
   }
}
